#!/usr/bin/env python
# -*- coding: ISO8859-1 -*-

############################################################################
###Programme d'envoi des donnees IPMI du bullion a vCops
###Code par Guillaume Bellosguardo / Virtualisation Competence Center (Pierre Fumery) - Bull Grenoble
###guillaume.bellosguardo@ext.bull.net
###
### Se rfrer a la documentation
### VERSION 1.0

### LIBRAIRIES ###

import os
import StringIO
import time
import string
import httplibcustom as httplib2
import logging
import logging.handlers
import ConfigParser
import socket


### FONCTIONS INTERNES ###

### loads the configuration from bulliontovcop.conf
def loadConfig():
	try :
		global globalconf, bullionsconf, metricsconf
		configuration = ConfigParser.ConfigParser()
		configuration.read("bulliontovcop.conf")
		globalconf = dict(configuration.items("DEFAULT"))
		bullionsconf = dict()
		for bullion in configuration.sections():
			bullionsconf[bullion]=dict(configuration.items(bullion))
		configuration.read("metrics.conf")
		metricsconf = dict(configuration.items("DEFAULT"))
		for metric in metricsconf :
			if metricsconf[metric].lower()=='on':
				metricsconf[metric]=True
			else:
				metricsconf[metric]=False
	except Exception as int:
		print 'ERROR LOADING CONFIG ' + str(inst)
	

### Initializes the logger
def initLog():
	logger = logging.getLogger('logger1')
	fh = logging.handlers.RotatingFileHandler('bulliontovcop.log', maxBytes=2000000, backupCount=5)
	numeric_level = getattr(logging, globalconf['loglevel'].upper(), None)
	if not isinstance(numeric_level, int):
		raise ValueError('Invalid log level: %s' % globalconf['loglevel'])
	logger.setLevel(numeric_level)
	fh.setLevel(numeric_level)
	formatter= logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
	fh.setFormatter(formatter)
	logger.addHandler(fh)
	return logger
	
### checks the configuration
def checkOptions():
	global selfHealth
	for required in RequiredGeneralOptions:
		if required not in globalconf or globalconf[required]=='' :
			log.critical('Required general option ' +required+ ' is missing.')
			return required
	if globalconf['bsmpath'][0] != '/' or globalconf['bsmpath'][-1] != '/' :
			log.critical('invalid bsmpath in bulliontovcop.conf')
			return 'bsmpath'
	for bullion in bullionsconf :
		for required in RequiredBullionOptions :
			if required not in bullionsconf[bullion] or bullionsconf[bullion][required]=='':
				log.critical('Required bullion option ' +required+ ' is missing.')
				selfHealth -= 80
				return required
	log.info('Configuration seems OK')
	return 0
	
### get ipmi answer 'data' of the bullion 'bullion'
def getDataFromBullion(bullion):
	if 'id' not in bullionsconf[bullion] :
		try:
			moduleID= os.popen(globalconf['bsmpath'] +"bsmGetModuleLevelInfo.sh -H "+bullionsconf[bullion]['ip']+" -u "+bullionsconf[bullion]['username']+" -p "+bullionsconf[bullion]['password'], "r").read()
			moduleID= int(moduleID.splitlines()[0][-1])*2 #Conversion vers le dernier digit de ce qu'affiche IPMI
			bullionsconf[bullion]['id']=moduleID
			log.debug('Id for bullion ' + bullion + ' has been successfuly initialized to ' + str(moduleID))
		except:
			return 'ERROR on module ID initialization -' + moduleID
	return os.popen(globalconf['bsmpath'] +"bsmsensor.sh -m ipmilan -H "+bullionsconf[bullion]['ip']+" -u "+bullionsconf[bullion]['username']+" -p "+bullionsconf[bullion]['password'], "r").read()

### creates vcops-formatted data 'formatted' from raw ipmi sensor data 'data'
def formatData(data, thetime, bullion):
	#Constantes L= lower U = upper C= critical NR= non recoverable NC = non critical
	#ipmitool classique NAME=0;VALUE=1;UNIT=2;STATUS=3;LNR=4;LC=5;LNC=6;UNC=7;UC=8;UNR=9;
	ADDR=0;NAME=1;OWNER=2;VALUE=3;UNIT=4;STATUS=5;LNR=6;LC=7;LNC=8;UNC=9;UC=10;UNR=11;
	global selfHealth	
	fileString = StringIO.StringIO(data)
	fileString.readline() #Retrait des deux premires lignes d'entte
	fileString.readline()
	lines = fileString.readlines()
	readyToBuild=[]
	
	for uline in lines :													#Pour chaque ligne de la data IPMI
		line = string.split(uline, '|') 								#Separation des champs de la ligne
		line = [x.replace(' ','') for x in line]						#Retrait des espaces
		
		##Vrification de la validit de la ligne
		if str(line[OWNER][-1]) != str(bullionsconf[bullion]['id']) : 	#Filtrage des lignes indsirables (cas multimodule)
			continue
		if line[NAME].lower() in metricsconf :							#Verification de la prsence de la mtrique en ON dans metric.conf
			if not metricsconf[line[NAME].lower()] :
				continue
		else :
			if line[NAME] != '':										#Ajout dans metric.conf si mtrique absente
				log.warning('NEW METRIC DETECTED :' + line[NAME] +' Adding it to metrics.conf with status ON')
				metricsfile = open('metrics.conf', 'a')
				metricsfile.write('\n' + line[NAME] +'= on')
				metricsfile.close()
				metricsconf[line[NAME].lower()]=True
				selfHealth -=2
			else:														#Cas mtrique vide
				log.warning('Empty metric name - IPMI must have had a problem gathering metrics')
				selfHealth -=5
		
		##Formatage de la ligne en vue de son envoi  vCOps		
		formatted = line[NAME] +"|"
		formatted += line[UNIT]	
		formatted +=',0,NoValue,'
		formatted += thetime +','
		try :
			line[VALUE]=float(line[VALUE])
		except: #hex or invalid (na)
				try:
					#La BMC bullion remplace les valeurs hexa par 'na' et place la valeur relle dans la colonne status...
					line[VALUE] = int(line[STATUS], 16)/256
				except Exception as inst : # invalid (na/na)
					line[VALUE]=-1
		formatted += str(line[VALUE])+',,0'		#La valeur de la mesure
		
		readyToBuild.append(formatted)
		
	return readyToBuild

### builds vcops-formatted message 'msg' from 'bullion' bullion's name and 'toBuild' formatted data
def buildMsg(toBuild, bullion): 
	msg =""
	msg += bullionsconf[bullion]['resourcename']+","
	msg += bullionsconf[bullion]['adapterkindkey']+","
	msg += bullionsconf[bullion]['resourcekindkey']+",,"
	msg += bullionsconf[bullion]['resourcedescription']+","
	msg += globalconf['timeinterval']+",false,\n"
	for line in toBuild :
		msg += line + '\n'
	return msg

### Sends formatted message 'msg' to vCOps
def sendMsg(msg): 
	username = globalconf['vcopusername']
	password = globalconf['vcoppassword']
	http = httplib2.Http(disable_ssl_certificate_validation=True)
	http.add_credentials(username, password)
	try :
		response = http.request(
			"https://"+globalconf['vcopip'] +"/HttpPostAdapter/OpenAPIServlet",
			"POST",
			msg
		)
		log.debug('Message sent. response : ' + response[1].rstrip())
		return response
	except socket.error as e:
		log.error('Connection to vCOps failed : ' + str(e))
		return False
	


### Self Health reporting to vCOps
def sendHealth():
	global selfHealth
	if(selfHealth < 0) :
		selfHealth = 0
	#msg construction
	msg="bullionAdapterHealth,httposttest,health,bullionAdapterHealthScore,"
	msg+=globalconf['timeinterval']+",false,\n"
	msg+="Health,4,NoValue,"+ str(int(time.time()*1000)) +","+str(selfHealth)+",,50\n"
	#msg sending
	if sendMsg(msg) :
		log.debug('Health submitted (' + str(selfHealth) +')')
	selfHealth =100
	

## RUN FUNCTION
## Load & Check config then
## Indefinitely 
##		get data from all the configured bullions
## 		Then push the wanted data to vCOps
## 		Also log warning and errors and calculate & push self health accordingly
## 		sleep for TimeInterval
def Run():
	global selfHealth
	missingOption = checkOptions()
	log.debug(' ******* Application initialized ******* ')
	
	if missingOption == 0:
		while True:
			try:
				#for each bullion configured, get, filter, format and send data
				for bullion in bullionsconf :
					#1 - get data from bullion
					ipmiRawData=getDataFromBullion(bullion)
					if ipmiRawData =='':
						#no data
						log.warning('No data from bullion ' + bullion + ' !')
						selfHealth -= 35
						continue
					if ipmiRawData.count('ERROR'):
						#erreur pendant l'execution
						log.warning("Error when trying to access " + bullion + " : " + ipmiRawData.splitlines()[0])
						selfHealth -= 35
						continue
					thetime = str(int(time.time()*1000))
					#try :
					#2 - Filter wanted data
					#msr = filtrerIPMI(ipmiRawData)
					
					#3 - Format data
					#tobuild=[]
					#for line in msr :
					#	readyMetric = formatData(line, thetime, bullion)
					#	if readyMetric != '' :
					#		tobuild.append(readyMetric)
					tobuild= formatData(ipmiRawData, thetime, bullion)
					#4 - Send data to vCOps
					result = sendMsg(buildMsg(tobuild, bullion))
					if result :
						if result[1].count('Complete')==1:
							log.info("Data submitted for " + bullion)
						else :
							log.error("failed to submit data for " + bullion + " See next line :")
							log.error(str(result[1]).rstrip())
							selfHealth -= 60
					else :
						#No connection to vCOps
						selfHealth -= 100
						log.critical("No connection to vCOps !")
						break

			except Exception as inst:
				log.error('Unexpected exception : ' + str(inst))
				selfHealth -= 100
				
			finally :
				#Send self health to vCOps
				sendHealth()				
				#Sleep for TimeInterval seconds
				log.debug('Ok - Sleeping')	
				time.sleep(60*float(globalconf['timeinterval']))
			
	else :
		log.critical('ERROR - invalid config file : ' + missingOption + ' is missing somewhere...')
		

		
### CONSTANT VARIABLES ###
RequiredBullionOptions=('ip', 'username', 'password', 'resourcename', 'adapterkindkey', 'resourcekindkey', 'resourcedescription')
RequiredGeneralOptions=('loglevel', 'timeinterval', 'vcopip', 'vcopusername', 'vcoppassword', 'bsmpath')

### GLOBAL VARIABLES & INITIALIZATION ###
#Configuration loading
globalconf=dict()
bullionsconf=dict()
metricsconf=dict()
loadConfig()
#Self Health initialization
selfHealth=100
#Log configuration
log = initLog()



### MAIN ###
if __name__ == '__main__':
	Run()
				


